<?php namespace IFWP;

final class Cloudinary {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function plugins_loaded(){
		add_filter('fl_builder_photo_sizes_select', [__CLASS__, 'fl_builder_photo_sizes_select']);
		add_filter('image_downsize', [__CLASS__, 'image_downsize'], 10, 3);
		add_filter('image_size_names_choose', [__CLASS__, 'image_size_names_choose']);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// private
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static $config = [], $image_sizes = [];

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static function meta_key($options = []){
        return 'ifwp_cloudinary_' . ifwp_md5($options);
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    private static function meta_value($id = 0, $options = []){
        return get_post_meta($id, self::meta_key($options), true);
    }

    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function auth(){
        if(self::$config){
			return self::$config;
		}
        $missing = [];
        if(!defined('IFWP_CLOUDINARY_API_KEY')){
			$missing[] = 'API Key';
        }
        if(!defined('IFWP_CLOUDINARY_API_SECRET')){
			$missing[] = 'API Secret';
        }
        if(!defined('IFWP_CLOUDINARY_CLOUD_NAME')){
			$missing[] = 'Cloud Name';
        }
        if(!empty($missing)){
            return ifwp_error(sprintf(__('Missing parameter(s): %s'), ifwp_implode_and($missing)) . '.');
        }
        if(!class_exists('Cloudinary')){
            $dir = ifwp_remote_lib('https://github.com/cloudinary/cloudinary_php/archive/refs/tags/1.20.0.zip', 'cloudinary_php-1.20.0');
            if(is_wp_error($dir)){
                return $dir;
            }
            require_once($dir . '/autoload.php');
        }
		self::$config = [
            'api_key' => IFWP_CLOUDINARY_API_KEY,
            'api_secret' => IFWP_CLOUDINARY_API_SECRET,
            'cloud_name' => IFWP_CLOUDINARY_CLOUD_NAME,
        ];
        return \Cloudinary::config(self::$config);
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function add_image_size($name = '', $options = []){
        $config = self::auth();
		if(is_wp_error($config)){
			return $config;
		}
        $size = sanitize_title($name);
        if(remove_image_size($size)){
            if(isset(\IFWP::$image_sizes[$size])){
                unset(\IFWP::$image_sizes[$size]);
            }
        }
        add_image_size($size); // fake - required
        self::$image_sizes[$size] = [
            'name' => $name,
            'options' => $options,
        ];
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function fl_builder_photo_sizes_select($sizes){
		$cl_image_sizes = self::$image_sizes;
		if($cl_image_sizes){
			if(isset($sizes['full'])){
				$id = ifwp_attachment_url_to_postid($sizes['full']['url']);
				if($id){
					foreach($cl_image_sizes as $size => $args){
						if(isset($sizes[$size])){
							continue;
						}
						$meta_key = self::meta_key($args['options']);
						$result = get_post_meta($id, $meta_key, true);
						if(!$result){
							continue;
						}
						$filename = $result['public_id'] . '.' . $result['format'];
						$height = $result['height'];
						$url = $result['secure_url'];
						$width = $result['width'];
						if(!$filename or !$height or !$url or !$width){
							continue;
						}
						$sizes[$size] = [
							'filename' => $filename,
							'height' => $height,
							'url' => $url,
							'width' => $width,
						];
					}
				}
			}
		}
		uasort($sizes, function($a, $b){
			if($a['width'] === $b['width']){
				if($a['height'] === $b['height']){
					return 0;
				}
				if($a['height'] < $b['height']){
					return -1;
				}
				return 1;
			}
			if($a['width'] < $b['width']){
				return -1;
			}
			return 1;
		});
		return $sizes;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function image_downsize($out, $id, $size){
        if($out){
            return $out;
        }
        if(!wp_attachment_is_image($id)){
            return false;
        }
        if(!is_scalar($size)){
            return false;
        }
        if(!isset(self::$image_sizes[$size])){
            return false;
        }
        $options = self::$image_sizes[$size]['options'];
        $result = self::meta_value($id, $options);
        if(!$result){
            $result = self::upload_attachment($id, $size);
            if(is_wp_error($result)){
                return false;
            }
        }
        $url = $result['secure_url'];
        $width = $result['width'];
        $height = $result['height'];
        if(!$url or !$width or !$height){
            return false;
        }
        return [$url, $width, $height, true];
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function image_size_names_choose($sizes){
        foreach(self::$image_sizes as $size => $args){
            $sizes[$size] = $args['name'];
        }
        return $sizes;
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function upload_file($file = '', $options = []){
        if(!@file_exists($file)){
            return ifwp_error(__('File doesn&#8217;t exist?'), $file);
        }
        if(!class_exists('Cloudinary')){
            $dir = ifwp_remote_lib('https://github.com/cloudinary/cloudinary_php/archive/refs/tags/1.20.0.zip', 'cloudinary_php-1.20.0');
            if(is_wp_error($dir)){
                return $dir;
            }
            require_once($dir . '/autoload.php');
        }
        try {
            $result = \Cloudinary\Uploader::upload($file, $options);
        } catch (Throwable $t){
            return ifwp_error($t->getMessage());
        } catch (Exception $e){
            return ifwp_error($e->getMessage());
        }
        return $result;
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    public static function upload_attachment($id = 0, $size = ''){
        if(!wp_attachment_is_image($id)){
            return ifwp_error(__('File is not an image.'));
        }
        if(!is_scalar($size)){
            return ifwp_error(sprintf(__('Invalid parameter(s): %s'), 'size') . '.');
        }
        $size = sanitize_title($size);
        if(!isset(self::$image_sizes[$size])){
            return ifwp_error(sprintf(__('Invalid parameter(s): %s'), 'size') . '.');
        }
        $options = self::$image_sizes[$size]['options'];
        $result = self::meta_value($id, $options);
        if($result){
            return $result;
        }
		$metadata = wp_get_attachment_metadata($id);
		if((int) $metadata['filesize'] < 20 * MB_IN_BYTES){
			$file = get_attached_file($id);
		} else {
			$sizes = $metadata['sizes'];
			$sizes = wp_list_pluck($sizes, 'filesize', 'file');
			arsort($sizes);
			$attachment_url = wp_get_attachment_url($id);
			$base_url = str_replace(wp_basename($attachment_url), '', $attachment_url);
			$file_url = '';
			foreach($sizes as $file => $filesize){
				if((int) $filesize > 20 * MB_IN_BYTES){
					continue;
				}
				$file_url = $base_url . $file;
				break;
			}
			if(!$file_url){
				return ifwp_error(__('Error') . '.');
			}
			$upload_dir = wp_get_upload_dir();
			$file = str_replace($upload_dir['baseurl'], $upload_dir['basedir'], $file_url);
		}
        $result = self::upload_file($file, $options);
        if(is_wp_error($result)){
            return $result;
        }
        update_post_meta($id, self::meta_key($options), $result);
        return $result;
    }

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}
