<?php namespace IFWP;

final class Uploads {

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//
	// public
	//
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function check_upload_dir($path = ''){
		$path = realpath($path);
		if(false === $path){
			$error_msg = __('File does not exist! Please double check the name and try again.');
			return ifwp_error($error_msg);
		}
		$path = wp_normalize_path($path);
		$upload_dir = wp_get_upload_dir();
		$basedir = wp_normalize_path(realpath($upload_dir['basedir']));
		if(0 !== strpos($path, $basedir)){
			$error_msg = sprintf(__('Unable to locate needed folder (%s).'), __('The uploads directory'));
			return ifwp_error($error_msg);
		}
		return $path;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function check_upload_size($file_size = 0){ // wp-includes/rest-api/endpoints/class-wp-rest-attachments-controller.php
		if(!is_multisite()){
			return true;
		}
		if(get_site_option('upload_space_check_disabled')){
			return true;
		}
		$space_left = get_upload_space_available();
		if($space_left < $file_size){
			$error_msg = sprintf(__('Not enough space to upload. %s KB needed.'), number_format(($file_size - $space_left) / KB_IN_BYTES));
			return ifwp_error($error_msg);
		}
		if($file_size > (KB_IN_BYTES * get_site_option('fileupload_maxk', 1500))){
			$error_msg = sprintf(__('This file is too big. Files must be less than %s KB in size.'), get_site_option('fileupload_maxk', 1500));
			return ifwp_error($error_msg);
		}
		if(!function_exists('upload_is_user_over_quota')){
			require_once(ABSPATH . 'wp-admin/includes/ms.php'); // Include multisite admin functions to get access to upload_is_user_over_quota().
		}
		if(upload_is_user_over_quota(false)){
			$error_msg = __('You have used your space quota. Please delete files before uploading.');
			return ifwp_error($error_msg);
		}
		return true;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function handle_file($file = [], $path = '', $mimes = null){
		if(empty($file)){
			$error_msg = __('No data supplied.');
			return ifwp_error($error_msg);
		}
		$keys = ['error', 'name', 'size', 'tmp_name', 'type'];
		foreach($keys as $key){
			if(isset($file[$key])){
				$file[$key] = (array) $file[$key];
			} else {
				$file[$key] = [];
			}
		}
		$count = count($file['tmp_name']);
		$files = [];
		for($i = 0; $i < $count; $i ++){
			$files[$i] = [];
			foreach($keys as $key){
				if(isset($file[$key][$i])){
					$files[$i][$key] = $file[$key][$i];
				}
			}
		}
		$uploaded_files = [];
		foreach($files as $index => $file){
			$uploaded_files[$index] = self::handle_upload($file, $path, $mimes);
		}
		return $uploaded_files;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function handle_files($files = [], $path = '', $mimes = null){ // $_FILES
		if(empty($files)){
			$error_msg = __('No data supplied.');
			return ifwp_error($error_msg);
		}
		$uploaded_files = [];
		foreach($files as $key => $file){
			$uploaded_files[$key] = self::handle_file($file, $path, $mimes);
		}
		return $uploaded_files;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function handle_upload($file = [], $path = '', $mimes = null){
		if(empty($file)){
			$error_msg = __('No data supplied.');
			return ifwp_error($error_msg);
		}
		$file = shortcode_atts([
			'error' => 0,
			'name' => '',
			'size' => 0,
			'tmp_name' => '',
			'type' => '',
		], $file);
		$uploaded_file = self::test_uploaded_file($file['tmp_name']);
		if(is_wp_error($uploaded_file)){
			return $uploaded_file;
		}
		$error = self::test_error($file['error']);
		if(is_wp_error($error)){
			return $error;
		}
		$size = self::test_size($file['size']);
		if(is_wp_error($size)){
			return $size;
		}
		$filename = self::test_type($file['tmp_name'], $file['name'], $mimes);
		if(is_wp_error($filename)){
			return $filename;
		}
		$size_check = self::check_upload_size($file['size']);
		if(is_wp_error($size_check)){
			return $size_check;
		}
		if($path){
			$upload_dir = self::check_upload_dir($path);
			if(is_wp_error($upload_dir)){
				return $upload_dir;
			}
			if(@is_file($upload_dir)){
				$upload_dir = dirname($upload_dir);
			}
			$path = $upload_dir;
		} else {
			$upload_dir = wp_upload_dir();
			$path = $upload_dir['path'];
		}
		$filename = apply_filters('ifwp_handle_upload_file_name', $filename, $file);
		$filename = wp_unique_filename($path, $filename);
		$new_file = path_join($path, $filename);
		$move_new_file = @move_uploaded_file($file['tmp_name'], $new_file);
		if(false === $move_new_file){
			$error_path = $path;
			if(0 === strpos($path, ABSPATH)){
				$error_path = str_replace(ABSPATH, '', $path);
			}
			$error_msg = sprintf(__('The uploaded file could not be moved to %s.'), $error_path);
			return ifwp_error($error_msg);
		}
		$stat = stat(dirname($new_file));
		$perms = $stat['mode'] & 0000666;
		chmod($new_file, $perms); // Set correct file permissions.
		if(is_multisite()){
			clean_dirsize_cache($new_file);
		}
		return $new_file;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function test_error($error = 0){ // A successful upload will pass this test.
		$upload_error_strings = [false, sprintf(__('The uploaded file exceeds the %1$s directive in %2$s.'), 'upload_max_filesize', 'php.ini'), sprintf(__('The uploaded file exceeds the %s directive that was specified in the HTML form.'), 'MAX_FILE_SIZE'), __('The uploaded file was only partially uploaded.'), __('No file was uploaded.'), '', __('Missing a temporary folder.'), __('Failed to write file to disk.'), __('File upload stopped by extension.')]; // Courtesy of php.net, the strings that describe the error indicated in $_FILES[{form field}]['error'].
		if($error > 0){
			if(empty($upload_error_strings[$error])){
				$error_msg = __('Something went wrong.');
			} else {
				$error_msg = $upload_error_strings[$error];
			}
			return ifwp_error($error_msg);
		}
		return true;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function test_size($file_size = 0){ // A non-empty file will pass this test.
		if(0 === $file_size){
			if(is_multisite()){
				$error_msg = __('File is empty. Please upload something more substantial.');
			} else {
				$error_msg = sprintf(__('File is empty. Please upload something more substantial. This error could also be caused by uploads being disabled in your %1$s file or by %2$s being defined as smaller than %3$s in %1$s.'), 'php.ini', 'post_max_size', 'upload_max_filesize');
			}
			return ifwp_error($error_msg);
		}
		return true;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function test_type($tmp_name = '', $name = '', $mimes = null){ // A correct MIME type will pass this test.
		$wp_filetype = wp_check_filetype_and_ext($tmp_name, $name, $mimes);
		$ext = empty($wp_filetype['ext']) ? '' : $wp_filetype['ext'];
		$type = empty($wp_filetype['type']) ? '' : $wp_filetype['type'];
		$proper_filename = empty($wp_filetype['proper_filename']) ? '' : $wp_filetype['proper_filename']; // Check to see if wp_check_filetype_and_ext() determined the filename was incorrect.
		if($proper_filename){
			$name = $proper_filename;
		}
		if((!$type or !$ext) and !current_user_can('unfiltered_upload')){
			$error_msg = __('Sorry, you are not allowed to upload this file type.');
			return ifwp_error($error_msg);
		}
		return $name;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public static function test_uploaded_file($tmp_name = ''){ // A properly uploaded file will pass this test.
		if(!is_uploaded_file($tmp_name)){
			$error_msg = __('Specified file failed upload test.');
			return ifwp_error($error_msg);
		}
		return true;
	}

	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

}
